@@ -1,5 +1,4 @@
 import mon  # noqa
-from ceph_deploy.hosts.centos import pkg  # noqa
 from ceph_deploy.hosts.centos.install import repo_install  # noqa
 from install import install, mirror_install  # noqa
 from uninstall import uninstall  # noqa
@@ -12,14 +11,17 @@ distro = None
 release = None
 codename = None
 
-def choose_init():
+def choose_init(module):
     """
     Select a init system
 
     Returns the name of a init system (upstart, sysvinit ...).
     """
-    return 'systemd'
 
+    if not module.conn.remote_module.path_exists("/usr/lib/systemd/system/ceph.target"):
+        return 'sysvinit'
+
+    return 'systemd'
 
 def get_packager(module):
     if module.normalized_release.int_major >= 22:
