@@ -41,18 +41,18 @@ def machine_type():
     return platform.machine()
 
 
-def write_sources_list(url, codename, filename='ceph.list'):
-    """add deb repo to sources.list"""
+def write_sources_list(url, codename, filename='ceph.list', mode=0644):
+    """add deb repo to /etc/apt/sources.list.d/"""
     repo_path = os.path.join('/etc/apt/sources.list.d', filename)
-    with file(repo_path, 'w') as f:
-        f.write('deb {url} {codename} main\n'.format(
-                url=url,
-                codename=codename,
-                ))
+    content = 'deb {url} {codename} main\n'.format(
+        url=url,
+        codename=codename,
+    )
+    write_file(repo_path, content, mode)
 
 
 def write_yum_repo(content, filename='ceph.repo'):
-    """set the contents of repo file to /etc/yum.repos.d/"""
+    """add yum repo file in /etc/yum.repos.d/"""
     repo_path = os.path.join('/etc/yum.repos.d', filename)
     write_file(repo_path, content)
 
@@ -123,7 +123,7 @@ def write_conf(cluster, conf, overwrite):
         raise RuntimeError(err_msg)
 
 
-def write_keyring(path, key):
+def write_keyring(path, key, uid=-1, gid=-1):
     """ create a keyring file """
     # Note that we *require* to avoid deletion of the temp file
     # otherwise we risk not being able to copy the contents from
@@ -133,27 +133,30 @@ def write_keyring(path, key):
     tmp_file.close()
     keyring_dir = os.path.dirname(path)
     if not path_exists(keyring_dir):
-        makedir(keyring_dir)
+        makedir(keyring_dir, uid, gid)
     shutil.move(tmp_file.name, path)
 
 
-def create_mon_path(path):
+def create_mon_path(path, uid=-1, gid=-1):
     """create the mon path if it does not exist"""
     if not os.path.exists(path):
         os.makedirs(path)
+        os.chown(path, uid, gid);
 
 
-def create_done_path(done_path):
+def create_done_path(done_path, uid=-1, gid=-1):
     """create a done file to avoid re-doing the mon deployment"""
     with file(done_path, 'w'):
         pass
+    os.chown(done_path, uid, gid);
 
 
-def create_init_path(init_path):
+def create_init_path(init_path, uid=-1, gid=-1):
     """create the init path if it does not exist"""
     if not os.path.exists(init_path):
         with file(init_path, 'w'):
             pass
+        os.chown(init_path, uid, gid);
 
 
 def append_to_file(file_path, contents):
@@ -161,6 +164,11 @@ def append_to_file(file_path, contents):
     with open(file_path, 'a') as f:
         f.write(contents)
 
+def path_getuid(path):
+    return os.stat(path).st_uid
+
+def path_getgid(path):
+    return os.stat(path).st_gid
 
 def readline(path):
     with open(path) as _file:
@@ -179,7 +187,7 @@ def listdir(path):
     return os.listdir(path)
 
 
-def makedir(path, ignored=None):
+def makedir(path, ignored=None, uid=-1, gid=-1):
     ignored = ignored or []
     try:
         os.makedirs(path)
@@ -189,20 +197,30 @@ def makedir(path, ignored=None):
         else:
             # re-raise the original exception
             raise
+    else:
+        os.chown(path, uid, gid);
 
 
 def unlink(_file):
     os.unlink(_file)
 
 
-def write_monitor_keyring(keyring, monitor_keyring):
+def write_monitor_keyring(keyring, monitor_keyring, uid=-1, gid=-1):
     """create the monitor keyring file"""
-    write_file(keyring, monitor_keyring)
+    write_file(keyring, monitor_keyring, 0600, None, uid, gid)
 
 
-def write_file(path, content):
-    with file(path, 'w') as f:
+def write_file(path, content, mode=0644, directory=None, uid=-1, gid=-1):
+    if directory:
+        if path.startswith("/"):
+            path = path[1:]
+        path = os.path.join(directory, path)
+    if os.path.exists(path):
+        # Delete file in case we are changing its mode
+        os.unlink(path)
+    with os.fdopen(os.open(path, os.O_WRONLY | os.O_CREAT, mode), 'w') as f:
         f.write(content)
+    os.chown(path, uid, gid)
 
 
 def touch_file(path):
@@ -285,7 +303,7 @@ def make_mon_removed_dir(path, file_name):
     shutil.move(path, os.path.join('/var/lib/ceph/mon-removed/', file_name))
 
 
-def safe_mkdir(path):
+def safe_mkdir(path, uid=-1, gid=-1):
     """ create path if it doesn't exist """
     try:
         os.mkdir(path)
@@ -294,6 +312,20 @@ def safe_mkdir(path):
             pass
         else:
             raise
+    else:
+        os.chown(path, uid, gid)
+
+def safe_makedirs(path, uid=-1, gid=-1):
+    """ create path recursively if it doesn't exist """
+    try:
+        os.makedirs(path)
+    except OSError, e:
+        if e.errno == errno.EEXIST:
+            pass
+        else:
+            raise
+    else:
+        os.chown(path, uid, gid)
 
 
 def zeroing(dev):
@@ -313,6 +345,15 @@ def zeroing(dev):
         f.write(size*'\0')
 
 
+def enable_yum_priority_obsoletes(path="/etc/yum/pluginconf.d/priorities.conf"):
+    """Configure Yum priorities to include obsoletes"""
+    config = ConfigParser.ConfigParser()
+    config.read(path)
+    config.set('main', 'check_obsoletes', '1')
+    with open(path, 'wb') as fout:
+        config.write(fout)
+
+
 # remoto magic, needed to execute these functions remotely
 if __name__ == '__channelexec__':
     for item in channel:  # noqa
