@@ -1,5 +1,4 @@
 import mon  # noqa
-import pkg  # noqa
 from install import install, mirror_install, repo_install  # noqa
 from uninstall import uninstall  # noqa
 import logging
@@ -15,7 +14,7 @@ distro = None
 release = None
 codename = None
 
-def choose_init():
+def choose_init(module):
     """
     Select a init system
 
