@@ -116,7 +116,7 @@ def get_public_network_ip(ips, public_subnet):
 
 def new(args):
     if args.ceph_conf:
-        raise RuntimeError('will not create a ceph conf file if attemtping to re-use with `--ceph-conf` flag')
+        raise RuntimeError('will not create a Ceph conf file if attemtping to re-use with `--ceph-conf` flag')
     LOG.debug('Creating new cluster named %s', args.cluster)
     cfg = conf.ceph.CephConf()
     cfg.add_section('global')
@@ -143,6 +143,14 @@ def new(args):
         # Now get the non-local IPs from the remote node
         distro = hosts.get(host, username=args.username)
         remote_ips = net.ip_addresses(distro.conn)
+
+        # custom cluster names on sysvinit hosts won't work
+        if distro.init == 'sysvinit' and args.cluster != 'ceph':
+            LOG.error('custom cluster names are not supported on sysvinit hosts')
+            raise exc.ClusterNameError(
+                'host %s does not support custom cluster names' % host
+            )
+
         distro.conn.exit()
 
         # Validate subnets if we received any
@@ -182,9 +190,6 @@ def new(args):
     cfg.set('global', 'auth service required', 'cephx')
     cfg.set('global', 'auth client required', 'cephx')
 
-    # http://tracker.newdream.net/issues/3138
-    cfg.set('global', 'filestore xattr use omap', 'true')
-
     path = '{name}.conf'.format(
         name=args.cluster,
         )
@@ -211,18 +216,21 @@ def new_mon_keyring(args):
     keypath = '{name}.mon.keyring'.format(
         name=args.cluster,
         )
-
+    oldmask = os.umask(077)
     LOG.debug('Writing monitor keyring to %s...', keypath)
-    tmp = '%s.tmp' % keypath
-    with file(tmp, 'w') as f:
-        f.write(mon_keyring)
     try:
-        os.rename(tmp, keypath)
-    except OSError as e:
-        if e.errno == errno.EEXIST:
-            raise exc.ClusterExistsError(keypath)
-        else:
-            raise
+        tmp = '%s.tmp' % keypath
+        with open(tmp, 'w', 0600) as f:
+            f.write(mon_keyring)
+        try:
+            os.rename(tmp, keypath)
+        except OSError as e:
+            if e.errno == errno.EEXIST:
+                raise exc.ClusterExistsError(keypath)
+            else:
+                raise
+    finally:
+        os.umask(oldmask)
 
 
 @priority(10)
