@@ -9,8 +9,8 @@ from textwrap import dedent
 
 from cStringIO import StringIO
 
-from ceph_deploy import conf, exc, hosts
-from ceph_deploy.util import constants, system
+from ceph_deploy import conf, exc, hosts, mon
+from ceph_deploy.util import constants, system, packages
 from ceph_deploy.cliutil import priority
 from ceph_deploy.lib import remoto
 
@@ -30,7 +30,7 @@ def get_bootstrap_osd_key(cluster):
         raise RuntimeError('bootstrap-osd keyring not found; run \'gatherkeys\'')
 
 
-def create_osd(conn, cluster, key):
+def create_osd_keyring(conn, cluster, key):
     """
     Run on osd node, writes the bootstrap key if not there yet.
     """
@@ -42,16 +42,6 @@ def create_osd(conn, cluster, key):
         logger.warning('osd keyring does not exist yet, creating one')
         conn.remote_module.write_keyring(path, key)
 
-    return remoto.process.run(
-        conn,
-        [
-            'udevadm',
-            'trigger',
-            '--subsystem-match=block',
-            '--action=add',
-        ],
-    )
-
 
 def osd_tree(conn, cluster):
     """
@@ -117,8 +107,9 @@ def osd_status_check(conn, cluster):
     Note how the booleans are actually strings, so we need to take that into
     account and fix it before returning the dictionary. Issue #8108
     """
+    ceph_executable = system.executable_path(conn, 'ceph')
     command = [
-        'ceph',
+        ceph_executable,
         '--cluster={cluster}'.format(cluster=cluster),
         'osd',
         'stat',
@@ -193,33 +184,35 @@ def prepare_disk(
         disk,
         journal,
         activate_prepared_disk,
+        init,
         zap,
         fs_type,
         dmcrypt,
-        dmcrypt_dir):
+        dmcrypt_dir,
+        storetype):
     """
     Run on osd node, prepares a data disk for use.
     """
+    ceph_disk_executable = system.executable_path(conn, 'ceph-disk')
     args = [
-        'ceph-disk',
+        ceph_disk_executable,
         '-v',
         'prepare',
         ]
     if zap:
         args.append('--zap-disk')
-    if fs_type:
-        if fs_type not in ('btrfs', 'ext4', 'xfs'):
-            raise argparse.ArgumentTypeError(
-                "FS_TYPE must be one of 'btrfs', 'ext4' or 'xfs'")
-        args.extend(['--fs-type', fs_type])
     if dmcrypt:
         args.append('--dmcrypt')
         if dmcrypt_dir is not None:
             args.append('--dmcrypt-key-dir')
             args.append(dmcrypt_dir)
+    if storetype:
+        args.append('--' + storetype)
     args.extend([
         '--cluster',
         cluster,
+        '--fs-type',
+        fs_type,
         '--',
         disk,
     ])
@@ -233,15 +226,15 @@ def prepare_disk(
     )
 
     if activate_prepared_disk:
-        return remoto.process.run(
-            conn,
-            [
-                'udevadm',
-                'trigger',
-                '--subsystem-match=block',
-                '--action=add',
-            ],
-        )
+        # we don't simply run activate here because we don't know
+        # which partition ceph-disk prepare created as the data
+        # volume.  instead, we rely on udev to do the activation and
+        # just give it a kick to ensure it wakes up.  we also enable
+        # ceph.target, the other key piece of activate.
+        if init == 'systemd':
+            system.enable_service(conn, "ceph.target")
+        elif init == 'sysvinit':
+            system.enable_service(conn, "ceph")
 
 
 def exceeds_max_osds(args, reasonable=20):
@@ -293,7 +286,11 @@ def prepare(args, cfg, activate_prepared_disk):
             if disk is None:
                 raise exc.NeedDiskError(hostname)
 
-            distro = hosts.get(hostname, username=args.username)
+            distro = hosts.get(
+                hostname,
+                username=args.username,
+                callbacks=[packages.ceph_is_installed]
+            )
             LOG.info(
                 'Distro info: %s %s %s',
                 distro.name,
@@ -313,21 +310,27 @@ def prepare(args, cfg, activate_prepared_disk):
                     args.overwrite_conf
                 )
 
-                create_osd(distro.conn, args.cluster, key)
+                create_osd_keyring(distro.conn, args.cluster, key)
 
             LOG.debug('Preparing host %s disk %s journal %s activate %s',
                       hostname, disk, journal, activate_prepared_disk)
 
+            storetype = None
+            if args.bluestore:
+                storetype = 'bluestore'
+
             prepare_disk(
                 distro.conn,
                 cluster=args.cluster,
                 disk=disk,
                 journal=journal,
                 activate_prepared_disk=activate_prepared_disk,
+                init=distro.init,
                 zap=args.zap_disk,
                 fs_type=args.fs_type,
                 dmcrypt=args.dmcrypt,
                 dmcrypt_dir=args.dmcrypt_key_dir,
+                storetype=storetype,
             )
 
             # give the OSD a few seconds to start
@@ -355,7 +358,11 @@ def activate(args, cfg):
 
     for hostname, disk, journal in args.disk:
 
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            callbacks=[packages.ceph_is_installed]
+        )
         LOG.info(
             'Distro info: %s %s %s',
             distro.name,
@@ -365,11 +372,12 @@ def activate(args, cfg):
 
         LOG.debug('activating host %s disk %s', hostname, disk)
         LOG.debug('will use init type: %s', distro.init)
-
+        
+        ceph_disk_executable = system.executable_path(distro.conn, 'ceph-disk')
         remoto.process.run(
             distro.conn,
             [
-                'ceph-disk',
+                ceph_disk_executable,
                 '-v',
                 'activate',
                 '--mark-init',
@@ -382,8 +390,10 @@ def activate(args, cfg):
         time.sleep(5)
         catch_osd_errors(distro.conn, distro.conn.logger, args)
 
-        if distro.is_el:
-            system.enable_service(distro.conn)
+        if distro.init == 'systemd':
+            system.enable_service(distro.conn, "ceph.target")
+        elif distro.init == 'sysvinit':
+            system.enable_service(distro.conn, "ceph")
 
         distro.conn.exit()
 
@@ -394,7 +404,11 @@ def disk_zap(args):
         if not disk or not hostname:
             raise RuntimeError('zap command needs both HOSTNAME and DISK but got "%s %s"' % (hostname, disk))
         LOG.debug('zapping %s on %s', disk, hostname)
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            callbacks=[packages.ceph_is_installed]
+        )
         LOG.info(
             'Distro info: %s %s %s',
             distro.name,
@@ -421,10 +435,11 @@ def disk_zap(args):
         if distro.normalized_name.startswith(('centos', 'red')):
             LOG.info('calling partx on zapped device %s', disk)
             LOG.info('re-reading known partitions will display errors')
+            partx_executable = system.executable_path(distro.conn, 'partx')
             remoto.process.run(
                 distro.conn,
                 [
-                    'partx',
+                    partx_executable,
                     '-a',
                     disk,
                 ],
@@ -432,10 +447,11 @@ def disk_zap(args):
 
         else:
             LOG.debug('Calling partprobe on zapped device %s', disk)
+            partprobe_executable = system.executable_path(distro.conn, 'partprobe')
             remoto.process.run(
                 distro.conn,
                 [
-                    'partprobe',
+                    partprobe_executable,
                     disk,
                 ],
             )
@@ -445,7 +461,11 @@ def disk_zap(args):
 
 def disk_list(args, cfg):
     for hostname, disk, journal in args.disk:
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            callbacks=[packages.ceph_is_installed]
+        )
         LOG.info(
             'Distro info: %s %s %s',
             distro.name,
@@ -466,20 +486,16 @@ def disk_list(args, cfg):
 
 
 def osd_list(args, cfg):
-    # FIXME: this portion should probably be abstracted. We do the same in
-    # mon.py
-    cfg = conf.ceph.load(args)
-    mon_initial_members = cfg.safe_get('global', 'mon_initial_members')
-    monitors = re.split(r'[,\s]+', mon_initial_members)
-
-    if not monitors:
-        raise exc.NeedHostError(
-            'could not find `mon initial members` defined in ceph.conf'
-        )
+    monitors = mon.get_mon_initial_members(args, error_on_empty=True, _cfg=cfg)
 
     # get the osd tree from a monitor host
     mon_host = monitors[0]
-    distro = hosts.get(mon_host, username=args.username)
+    distro = hosts.get(
+        mon_host,
+        username=args.username,
+        callbacks=[packages.ceph_is_installed]
+    )
+
     tree = osd_tree(distro.conn, args.cluster)
     distro.conn.exit()
 
@@ -665,47 +681,117 @@ def make(parser):
     parser.formatter_class = argparse.RawDescriptionHelpFormatter
     parser.description = sub_command_help
 
-    parser.add_argument(
-        'subcommand',
-        metavar='SUBCOMMAND',
-        choices=[
-            'list',
-            'create',
-            'prepare',
-            'activate',
-            'destroy',
-            ],
-        help='list, create (prepare+activate), prepare, activate, or destroy',
+    osd_parser = parser.add_subparsers(dest='subcommand')
+
+    osd_list = osd_parser.add_parser(
+        'list',
+        help='List OSD info from remote host(s)'
         )
-    parser.add_argument(
+    osd_list.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK[:JOURNAL]',
+        type=colon_separated,
+        help='remote host to list OSDs from'
+        )
+
+    osd_create = osd_parser.add_parser(
+        'create',
+        help='Create new Ceph OSD daemon by preparing and activating disk'
+        )
+    osd_create.add_argument(
+        '--zap-disk',
+        action='store_true',
+        help='destroy existing partition table and content for DISK',
+        )
+    osd_create.add_argument(
+        '--fs-type',
+        metavar='FS_TYPE',
+        choices=['xfs',
+                 'ext4',
+                 'btrfs'
+                 ],
+        default='xfs',
+        help='filesystem to use to format DISK (xfs, btrfs, or ext4)',
+        )
+    osd_create.add_argument(
+        '--dmcrypt',
+        action='store_true',
+        help='use dm-crypt on DISK',
+        )
+    osd_create.add_argument(
+        '--dmcrypt-key-dir',
+        metavar='KEYDIR',
+        default='/etc/ceph/dmcrypt-keys',
+        help='directory where dm-crypt keys are stored',
+        )
+    osd_create.add_argument(
+        '--bluestore',
+        action='store_true', default=None,
+        help='bluestore objectstore',
+        )
+    osd_create.add_argument(
         'disk',
         nargs='+',
         metavar='HOST:DISK[:JOURNAL]',
         type=colon_separated,
         help='host and disk to prepare',
         )
-    parser.add_argument(
+
+    osd_prepare = osd_parser.add_parser(
+        'prepare',
+        help='Prepare a disk for use as Ceph OSD by formatting/partitioning disk'
+        )
+    osd_prepare.add_argument(
         '--zap-disk',
-        action='store_true', default=None,
+        action='store_true',
         help='destroy existing partition table and content for DISK',
         )
-    parser.add_argument(
+    osd_prepare.add_argument(
         '--fs-type',
         metavar='FS_TYPE',
+        choices=['xfs',
+                 'ext4',
+                 'btrfs'
+                 ],
         default='xfs',
-        help='filesystem to use to format DISK (xfs, btrfs or ext4)',
+        help='filesystem to use to format DISK (xfs, btrfs, or ext4)',
         )
-    parser.add_argument(
+    osd_prepare.add_argument(
         '--dmcrypt',
-        action='store_true', default=None,
+        action='store_true',
         help='use dm-crypt on DISK',
         )
-    parser.add_argument(
+    osd_prepare.add_argument(
         '--dmcrypt-key-dir',
         metavar='KEYDIR',
         default='/etc/ceph/dmcrypt-keys',
         help='directory where dm-crypt keys are stored',
         )
+    osd_prepare.add_argument(
+        '--bluestore',
+        action='store_true', default=None,
+        help='bluestore objectstore',
+        )
+    osd_prepare.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK[:JOURNAL]',
+        type=colon_separated,
+        help='host and disk to prepare',
+        )
+
+    osd_activate = osd_parser.add_parser(
+        'activate',
+        help='Start (activate) Ceph OSD from disk that was previously prepared'
+        )
+    osd_activate.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK[:JOURNAL]',
+        type=colon_separated,
+        help='host and disk to activate',
+        )
     parser.set_defaults(
         func=osd,
         )
@@ -716,46 +802,86 @@ def make_disk(parser):
     """
     Manage disks on a remote host.
     """
-    parser.add_argument(
-        'subcommand',
-        metavar='SUBCOMMAND',
-        choices=[
-            'list',
-            'prepare',
-            'activate',
-            'zap',
-            ],
-        help='list, prepare, activate, zap',
+    disk_parser = parser.add_subparsers(dest='subcommand')
+
+    disk_zap = disk_parser.add_parser(
+        'zap',
+        help='destroy existing partition table and content for DISK',
         )
-    parser.add_argument(
+    disk_zap.add_argument(
         'disk',
         nargs='+',
         metavar='HOST:DISK',
         type=colon_separated,
-        help='host and disk (or path)',
+        help='host and disk'
         )
-    parser.add_argument(
+
+    disk_list = disk_parser.add_parser(
+        'list',
+        help='List disk info from remote host(s)'
+        )
+    disk_list.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK',
+        type=colon_separated,
+        help='remote host to list OSDs from'
+        )
+
+    disk_prepare = disk_parser.add_parser(
+        'prepare',
+        help='Prepare a disk for use as Ceph OSD by formatting/partitioning disk'
+        )
+    disk_prepare.add_argument(
         '--zap-disk',
-        action='store_true', default=None,
+        action='store_true',
         help='destroy existing partition table and content for DISK',
         )
-    parser.add_argument(
+    disk_prepare.add_argument(
         '--fs-type',
         metavar='FS_TYPE',
+        choices=['xfs',
+                 'ext4',
+                 'btrfs'
+                 ],
         default='xfs',
-        help='filesystem to use to format DISK (xfs, btrfs or ext4)'
+        help='filesystem to use to format DISK (xfs, btrfs, or ext4)',
         )
-    parser.add_argument(
+    disk_prepare.add_argument(
         '--dmcrypt',
-        action='store_true', default=None,
+        action='store_true',
         help='use dm-crypt on DISK',
         )
-    parser.add_argument(
+    disk_prepare.add_argument(
         '--dmcrypt-key-dir',
         metavar='KEYDIR',
         default='/etc/ceph/dmcrypt-keys',
         help='directory where dm-crypt keys are stored',
         )
+    disk_prepare.add_argument(
+        '--bluestore',
+        action='store_true', default=None,
+        help='bluestore objectstore',
+        )
+    disk_prepare.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK',
+        type=colon_separated,
+        help='host and disk to prepare',
+        )
+
+    disk_activate = disk_parser.add_parser(
+        'activate',
+        help='Start (activate) Ceph OSD from disk that was previously prepared'
+        )
+    disk_activate.add_argument(
+        'disk',
+        nargs='+',
+        metavar='HOST:DISK',
+        type=colon_separated,
+        help='host and disk to activate',
+        )
     parser.set_defaults(
         func=disk,
         )
