@@ -29,4 +29,4 @@ default_components = namedtuple('DefaultComponents', ['rpm', 'deb'])
 default_components.rpm = tuple(_base_components + ['ceph-radosgw'])
 default_components.deb = tuple(_base_components + ['radosgw'])
 
-gpg_key_base_url = "git.ceph.com/?p=ceph.git;a=blob_plain;f=keys/"
+gpg_key_base_url = "download.ceph.com/keys/"
